@@ -68,7 +68,8 @@ Pittsburgh, PA.
 #define  OFF	    0
 #define  READY	    2
 #define  WAITING    3
-#define  MINSTACK   44
+#define  MINSTACK   32768 /* allocate at least 32KB for stacks */
+#define  STACKPAD   4096  /* pad any requested stack size to PAGE_SIZE */
 #ifndef MAX
 #define  MAX(a,b)   ((a) > (b) ? (a) : (b))
 #endif
@@ -78,7 +79,7 @@ Pittsburgh, PA.
 /* simple test, we normally want to mmap our stacks whenever possible, except
  * when we compile venus for Win95 */
 #undef MMAP_LWP_STACKS
-#if defined(HAVE_MMAP) && !defined(DJGPP) 
+#ifdef HAVE_MMAP
 #define MMAP_LWP_STACKS 1
 
 #if defined(__NetBSD__) || defined(__FreeBSD__)
@@ -96,6 +97,9 @@ struct QUEUE {
 static ucontext_t reaper; /* reaper context, see comments for lwp_Reaper() */
 static ucontext_t tracer; /* context for the for stack tracing thread */
 
+#define REAPER_STACKSIZE 32768
+#define TRACER_STACKSIZE 32768
+
 /* Invariant for runnable queues: The head of each queue points to the
 currently running process if it is in that queue, or it points to the
 next process in that queue that should run. */
@@ -456,9 +460,9 @@ int LWP_CreateProcess(void (*ep)(void *), int stacksize, int priority,
 	return LWP_ENOMEM;
 
     if (stacksize < MINSTACK)
-	stacksize = 1024;
+	stacksize = MINSTACK;
     else
-	stacksize = 4 * ((stacksize+3) / 4);
+	stacksize = (stacksize + (STACKPAD-1)) & ~(STACKPAD-1);
 
 #ifndef MMAP_LWP_STACKS
     stackptr = (char *) malloc(stacksize);
@@ -580,16 +584,14 @@ static void lwp_Tracer(void)
 
 static void init_contexts(void)
 {
-#define REAPER_STACKSIZE 8192
     getcontext(&reaper);
     reaper.uc_stack.ss_sp = malloc(REAPER_STACKSIZE);
     reaper.uc_stack.ss_size = REAPER_STACKSIZE;
     makecontext(&reaper, lwp_Reaper, 0);
 
-#define TRACER_STACKSIZE 16384
     getcontext(&tracer);
-    tracer.uc_stack.ss_sp = malloc(REAPER_STACKSIZE);
-    tracer.uc_stack.ss_size = REAPER_STACKSIZE;
+    tracer.uc_stack.ss_sp = malloc(TRACER_STACKSIZE);
+    tracer.uc_stack.ss_size = TRACER_STACKSIZE;
     makecontext(&tracer, lwp_Tracer, 0);
 }
 
@@ -735,6 +737,8 @@ static void Free_PCB(PROCESS pid)
 
     lwpdebug(0, "Entered Free_PCB");
 
+    IOMGR_Cancel(pid);
+
     lwpremove(pid, (pid->status==WAITING ?
 		    &blocked :
 		    &runnable[pid->priority]));
@@ -787,7 +791,7 @@ static int lwp_DispatchProcess(int dummy)
        the guard word at the front of the stack being damaged.
        WARNING!  This code assumes that stacks grow downward. */
     if (lwp_cpptr && lwp_cpptr->stackcheck &&
-        (lwp_cpptr->stackcheck != *(int *)lwp_cpptr->stack.ss_sp ||
+	(lwp_cpptr->stackcheck != *(int *)lwp_cpptr->stack.ss_sp ||
 	 (char *)&dummy < (char *)lwp_cpptr->stack.ss_sp))
     {
 	switch (lwp_overflowAction) {
@@ -872,7 +876,9 @@ static void Initialize_PCB(PROCESS temp, int priority, char *stack,
     temp->parm = arg;
 
     if (stack) {
-	temp->stackcheck = *(int *)stack;
+	/* seems to be broken on x86_64 */
+	/* *(int *)stack = temp->stackcheck = 0xBAD57ACC; */
+	temp->stackcheck = 0;
 
 	getcontext(&temp->ctx);
 	temp->stack.ss_sp = temp->ctx.uc_stack.ss_sp = stack;
@@ -882,6 +888,8 @@ static void Initialize_PCB(PROCESS temp, int priority, char *stack,
 					another runnable thread */
 	makecontext(&temp->ctx, (void (*)(void))func, 1, arg);
     }
+    else
+	temp->stackcheck = 0;
 
     lwpdebug(0, "Leaving Initialize_PCB\n");
 }
